@@ -27,7 +27,7 @@ class NameNodeCollector(TreeVisitor):
         self.name_nodes.append(node)
 
 
-class SkipDeclarations:
+class SkipDeclarations(object):
     """
     Variable and function declarations can often have a deep tree structure, 
     and yet most transformations don't need to descend to this depth. 
@@ -332,13 +332,20 @@ class InterpretCompilerDirectives(CythonTransform, SkipDeclarations):
 
     def __init__(self, context, compilation_option_overrides):
         super(InterpretCompilerDirectives, self).__init__(context)
-        self.compilation_option_overrides = compilation_option_overrides
+        self.compilation_option_overrides = {}
+        for key, value in compilation_option_overrides.iteritems():
+            self.compilation_option_overrides[unicode(key)] = value
         self.cython_module_names = set()
         self.option_names = {}
 
     # Set up processing and handle the cython: comments.
     def visit_ModuleNode(self, node):
         options = copy.copy(Options.option_defaults)
+        for key, value in self.compilation_option_overrides.iteritems():
+            if key in node.option_comments and node.option_comments[key] != value:
+                warning(node.pos, "Compiler directive differs between environment and file header; this will change "
+                        "in Cython 0.12. See http://article.gmane.org/gmane.comp.python.cython.devel/5233", 2)
+                break
         options.update(node.option_comments)
         options.update(self.compilation_option_overrides)
         self.options = options
@@ -658,6 +665,7 @@ class DecoratorTransform(CythonTransform, SkipDeclarations):
             rhs = decorator_result)
         return [func_node, reassignment]
 
+
 class AnalyseDeclarationsTransform(CythonTransform):
 
     basic_property = TreeFragment(u"""
@@ -670,14 +678,23 @@ property NAME:
 
     def __call__(self, root):
         self.env_stack = [root.scope]
+        # needed to determine if a cdef var is declared after it's used.
+        self.seen_vars_stack = []
         return super(AnalyseDeclarationsTransform, self).__call__(root)        
     
+    def visit_NameNode(self, node):
+        self.seen_vars_stack[-1].add(node.name)
+        return node
+
     def visit_ModuleNode(self, node):
+        self.seen_vars_stack.append(set())
         node.analyse_declarations(self.env_stack[-1])
         self.visitchildren(node)
+        self.seen_vars_stack.pop()
         return node
         
     def visit_FuncDefNode(self, node):
+        self.seen_vars_stack.append(set())
         lenv = node.create_local_scope(self.env_stack[-1])
         node.body.analyse_control_flow(lenv) # this will be totally refactored
         node.declare_arguments(lenv)
@@ -692,6 +709,7 @@ property NAME:
         self.env_stack.append(lenv)
         self.visitchildren(node)
         self.env_stack.pop()
+        self.seen_vars_stack.pop()
         return node
     
     # Some nodes are no longer needed after declaration
@@ -699,6 +717,8 @@ property NAME:
     # on these nodes in a seperate recursive process from the
     # enclosing function or module, so we can simply drop them.
     def visit_CDeclaratorNode(self, node):
+        # necessary to ensure that all CNameDeclaratorNodes are visited.
+        self.visitchildren(node)
         return node
     
     def visit_CTypeDefNode(self, node):
@@ -713,7 +733,19 @@ property NAME:
     def visit_CStructOrUnionDefNode(self, node):
         return None
 
+    def visit_CNameDeclaratorNode(self, node):
+        if node.name in self.seen_vars_stack[-1]:
+            entry = self.env_stack[-1].lookup(node.name)
+            if entry is None or entry.visibility != 'extern':
+                warning(node.pos, "cdef variable '%s' declared after it is used" % node.name, 2)
+        self.visitchildren(node)
+        return node
+
     def visit_CVarDefNode(self, node):
+
+        # to ensure all CNameDeclaratorNodes are visited.
+        self.visitchildren(node)
+
         if node.need_properties:
             # cdef public attributes may need type testing on 
             # assignment, so we create a property accesss
@@ -787,6 +819,9 @@ class AlignFunctionDefinitions(CythonTransform):
     def visit_DefNode(self, node):
         pxd_def = self.scope.lookup(node.name)
         if pxd_def:
+            if self.scope.is_c_class_scope and len(pxd_def.type.args) > 0:
+                # The self parameter type needs adjusting.
+                pxd_def.type.args[0].type = self.scope.parent_type
             if pxd_def.is_cfunction:
                 node = node.as_cfunction(pxd_def)
             else:
@@ -926,6 +961,18 @@ class TransformBuiltinMethods(EnvTransform):
                     error(node.function.pos, u"sizeof takes exactly one argument" % function)
                 else:
                     node = AmpersandNode(node.function.pos, operand=node.args[0])
+            elif function == 'cmod':
+                if len(node.args) != 2:
+                    error(node.function.pos, u"cmod takes exactly one argument" % function)
+                else:
+                    node = binop_node(node.function.pos, '%', node.args[0], node.args[1])
+                    node.cdivision = True
+            elif function == 'cdiv':
+                if len(node.args) != 2:
+                    error(node.function.pos, u"cmod takes exactly one argument" % function)
+                else:
+                    node = binop_node(node.function.pos, '/', node.args[0], node.args[1])
+                    node.cdivision = True
             else:
                 error(node.function.pos, u"'%s' not a valid cython language construct" % function)
         
