@@ -23,7 +23,24 @@ except NameError:
 possible_identifier = re.compile(ur"(?![0-9])\w+$", re.U).match
 nice_identifier = re.compile('^[a-zA-Z0-0_]+$').match
 
-class BufferAux:
+iso_c99_keywords = set(
+['auto', 'break', 'case', 'char', 'const', 'continue', 'default', 'do', 
+    'double', 'else', 'enum', 'extern', 'float', 'for', 'goto', 'if', 
+    'int', 'long', 'register', 'return', 'short', 'signed', 'sizeof', 
+    'static', 'struct', 'switch', 'typedef', 'union', 'unsigned', 'void', 
+    'volatile', 'while',
+    '_Bool', '_Complex'', _Imaginary', 'inline', 'restrict'])
+
+def c_safe_identifier(cname):
+    # There are some C limitations on struct entry names.
+    if ((cname[:2] == '__'
+         and not (cname.startswith(Naming.pyrex_prefix)
+                  or cname == '__weakref__'))
+        or cname in iso_c99_keywords):
+        cname = Naming.pyrex_prefix + cname
+    return cname
+
+class BufferAux(object):
     writable_needed = False
     
     def __init__(self, buffer_info_var, stridevars, shapevars,
@@ -36,7 +53,7 @@ class BufferAux:
     def __repr__(self):
         return "<BufferAux %r>" % self.__dict__
 
-class Entry:
+class Entry(object):
     # A symbol table entry in a Scope or ModuleNamespace.
     #
     # name             string     Python name of entity
@@ -158,7 +175,7 @@ class Entry:
         error(pos, "'%s' does not match previous declaration" % self.name)
         error(self.pos, "Previous declaration is here")
 
-class Scope:
+class Scope(object):
     # name              string             Unqualified name
     # outer_scope       Scope or None      Enclosing scope
     # entries           {string : Entry}   Python name to entry, non-types
@@ -194,7 +211,6 @@ class Scope:
     scope_prefix = ""
     in_cinclude = 0
     nogil = 0
-    directives = {}
     
     temp_prefix = Naming.pyrex_prefix
     
@@ -1252,6 +1268,8 @@ class StructOrUnionScope(Scope):
         # Add an entry for an attribute.
         if not cname:
             cname = name
+            if visibility == 'private':
+                cname = c_safe_identifier(cname)
         if type.is_cfunction:
             type = PyrexTypes.CPtrType(type)
         entry = self.declare(name, cname, type, pos, visibility)
@@ -1387,6 +1405,8 @@ class CClassScope(ClassScope):
                         % name)
             if not cname:
                 cname = name
+                if visibility == 'private':
+                    cname = c_safe_identifier(cname)
             entry = self.declare(name, cname, type, pos, visibility)
             entry.is_variable = 1
             self.var_entries.append(entry)
@@ -1456,7 +1476,8 @@ class CClassScope(ClassScope):
         if not args:
             error(pos, "C method has no self argument")
         elif not args[0].type.same_as(self.parent_type):
-            error(pos, "Self argument of C method does not match parent type")
+            error(pos, "Self argument (%s) of C method '%s' does not match parent type (%s)" %
+                  (args[0].type, name, self.parent_type))
         entry = self.lookup_here(name)
         if entry:
             if not entry.is_cfunction:
