@@ -2,7 +2,7 @@
 #   Pyrex -- Mac system interface
 #
 
-import os, sys, string
+import os, sys
 import aetools
 from aetools import TalkTo
 from StdSuites.Standard_Suite import Standard_Suite_Events as Standard_Suite
@@ -57,8 +57,8 @@ def do_toolserver_command(command):
     errn, stat, stdout, stderr = result
     if errn:
         raise CCompilerError("ToolServer error: %s" % errn)
-    stdout = string.replace(stdout, "\r", "\n")
-    stderr = string.replace(stderr, "\r", "\n")
+    stdout = stdout.replace("\r", "\n")
+    stderr = stderr.replace("\r", "\n")
     if stdout:
         #print "<<< Begin ToolServer StdOut >>>"
         sys.stderr.write(stdout)
@@ -85,7 +85,7 @@ def c_compile(c_file):
     command = "%s -opt %s -nomapcr -w off -r %s %s -o %s" % (
         c_compiler, 
         c_optimizations,
-        string.join(include_options),
+        ' '.join(include_options),
         c_file, 
         o_file, 
         #e_file
@@ -106,9 +106,9 @@ def c_link_list(obj_files):
     out_file = replace_suffix(obj_files[0], shared_lib_suffix)
     command = "%s -xm s -export all %s %s %s -o %s" % (
         c_linker, 
-        string.join(obj_files), 
+        ' '.join(obj_files), 
         pythoncore,
-        string.join(libraries),
+        ' '.join(libraries),
         out_file)
     stat = do_toolserver_command(command)
     if stat:
